package com.tzld.piaoquan.content.understanding.aop;

import com.alibaba.fastjson.JSON;
import com.google.common.base.Stopwatch;
import com.tzld.piaoquan.content.understanding.service.LoghubService;
import org.aspectj.lang.ProceedingJoinPoint;
import org.aspectj.lang.annotation.Around;
import org.aspectj.lang.annotation.Aspect;
import org.aspectj.lang.annotation.Pointcut;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import javax.servlet.http.HttpServletRequest;
import java.util.HashMap;
import java.util.Map;
import java.util.Objects;
import java.util.concurrent.TimeUnit;

@Aspect
@Component
public class RequestLogAspect {

    @Autowired
    private LoghubService loghubService;

    @Value("${aliyun.log.project}")
    private String aliyunLogProject;
    @Value("${aliyun.log.logstore.request}")
    private String aliyunLogLogstoreRequest;

    @Pointcut("execution(* com.tzld.piaoquan.content.understanding..*Controller.*(..)) && !within(com.tzld.piaoquan.content.understanding.controller.IndexController)")
    public void allController() {
    }

    @Around("allController()")
    public Object around(ProceedingJoinPoint joinPoint) throws Throwable {
        // 获取请求信息
        HttpServletRequest request = Objects.requireNonNull((ServletRequestAttributes) RequestContextHolder.getRequestAttributes()).getRequest();
        String uri = request.getRequestURI();
        String clientIp = request.getRemoteAddr();
        String queryString = request.getQueryString();
        String method = joinPoint.getSignature().getName();
        Stopwatch stopwatch = Stopwatch.createStarted();
        // 执行目标方法
        Object result = joinPoint.proceed();
        // 计算耗时
        long executionTime = stopwatch.stop().elapsed(TimeUnit.MILLISECONDS);
        Map<String, Object> data = new HashMap<>();
        data.put("uri", uri);
        data.put("clientIp", clientIp);
        data.put("method", method);
        data.put("executionTime", executionTime);
        data.put("queryString", queryString);
        data.put("requestBody", JSON.toJSONString(joinPoint.getArgs()));
        data.put("responseBody", JSON.toJSONString(result));
        loghubService.asyncSubmitLog(aliyunLogProject, aliyunLogLogstoreRequest, "", data);
        return result;
    }
}
