package com.tzld.piaoquan.content.understanding.common.base;

import java.util.UUID;

/**
 * Common Response
 */
public class CommonResponse<T> {

    private static final int SUCCESS_CODE = 0;
    private static final String SUCCESS_MSG = "success";

    /** 返回状态码，0 表示业务成功 */
    private int code = 0;
    /** 返回消息 */
    private String msg = "success";
    /** 业务成功时返回数据 */
    private T data;
    /** 重定向 */
    private String redirect;
    /**
     * 请求ID
     */
    private String requestId = UUID.randomUUID().toString();

    public boolean isSuccess() {
        return this.code == SUCCESS_CODE;
    }

    public static <T> CommonResponse<T> success() {
        CommonResponse<T> commonResponse = new CommonResponse<>();
        commonResponse.setCode(SUCCESS_CODE);
        commonResponse.setMsg(SUCCESS_MSG);
        return commonResponse;
    }

    public static <T> CommonResponse<T> success(T data) {
        CommonResponse<T> commonResponse = new CommonResponse<>();
        commonResponse.setCode(SUCCESS_CODE);
        commonResponse.setMsg(SUCCESS_MSG);
        commonResponse.setData(data);
        return commonResponse;
    }

    public static <T> CommonResponse<T> create() {
        return create(SUCCESS_CODE, SUCCESS_MSG, null);
    }
    
    public static <T> CommonResponse<T> create(T data) {
        return create(SUCCESS_CODE, SUCCESS_MSG, data);
    }

    public static <T> CommonResponse<T> create(int code, String msg) {
        return create(code, msg, null);
    }

    public static <T> CommonResponse<T> create(int code, String msg, T data) {
        CommonResponse<T> commonResponse = new CommonResponse<>();
        commonResponse.setCode(code);
        commonResponse.setMsg(msg);
        commonResponse.setData(data);
        return commonResponse;
    }

    public int getCode() {
        return code;
    }

    public void setCode(int code) {
        this.code = code;
    }

    public String getMsg() {
        return msg;
    }

    public void setMsg(String msg) {
        this.msg = msg;
    }

    public T getData() {
        return data;
    }

    public void setData(T data) {
        this.data = data;
    }

    public String getRedirect() {
        return redirect;
    }

    public void setRedirect(String redirect) {
        this.redirect = redirect;
    }

    public String getRequestId() {
        return requestId;
    }

    public void setRequestId(String requestId) {
        this.requestId = requestId;
    }
}
