package com.tzld.piaoquan.content.understanding.rocketmq.producer;

import com.alibaba.fastjson.JSON;
import com.tzld.piaoquan.content.understanding.model.dto.PipelineTaskMessageDTO;
import com.tzld.piaoquan.content.understanding.util.RetryUtil;
import org.apache.rocketmq.client.apis.message.MessageId;
import org.apache.rocketmq.client.apis.producer.SendReceipt;
import org.apache.rocketmq.client.core.RocketMQClientTemplate;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import java.time.Duration;
import java.util.Map;
import java.util.Objects;

/**
 * 内容理解任务 任务 生产者
 *
 * @author supeng
 */
@Component
public class ContentUnderstandingPipelineTaskProducer {
    @Autowired
    private RocketMQClientTemplate rocketMQClientTemplate;

    @Value("${rocketmq.producer.pipelinetask.topic:topic_content_understanding_pipeline_task}")
    private String pipelineTaskTopic;

    public MessageId syncSendNormalMessage(PipelineTaskMessageDTO messageDTO) {
        return RetryUtil.executeWithRetry(() -> {
            SendReceipt sendReceipt = rocketMQClientTemplate.syncSendNormalMessage(pipelineTaskTopic, JSON.toJSONString(messageDTO));
            if (Objects.isNull(sendReceipt) || Objects.isNull(sendReceipt.getMessageId())) {
                throw new RuntimeException("ContentUnderstandingPipelineTaskProducer syncSendNormalMessage failure messageDTO = " + messageDTO);
            }
            return sendReceipt.getMessageId();
        }, "ContentUnderstandingPipelineTaskProducer");
    }
}
