package com.tzld.piaoquan.content.understanding.service.impl;

import com.aliyun.odps.data.Record;
import com.aliyun.oss.model.ObjectMetadata;
import com.aliyun.oss.model.StorageClass;
import com.ctrip.framework.apollo.spring.annotation.ApolloJsonValue;
import com.google.common.base.Stopwatch;
import com.tzld.commons.aliyun.oss.AliyunOssManager;
import com.tzld.piaoquan.content.understanding.common.base.Constant;
import com.tzld.piaoquan.content.understanding.common.enums.ContentTypeEnum;
import com.tzld.piaoquan.content.understanding.common.enums.DataStatusEnum;
import com.tzld.piaoquan.content.understanding.common.enums.ExceptionEnum;
import com.tzld.piaoquan.content.understanding.common.exception.CommonException;
import com.tzld.piaoquan.content.understanding.dao.mapper.*;
import com.tzld.piaoquan.content.understanding.model.dto.ContentDTO;
import com.tzld.piaoquan.content.understanding.model.dto.TaskResultDTO;
import com.tzld.piaoquan.content.understanding.model.dto.VideoUnderstandingMessageDTO;
import com.tzld.piaoquan.content.understanding.model.param.*;
import com.tzld.piaoquan.content.understanding.model.po.*;
import com.tzld.piaoquan.content.understanding.model.vo.TaskResultVO;
import com.tzld.piaoquan.content.understanding.rocketmq.producer.OldVersionRemainVideoUnderstandingProducer;
import com.tzld.piaoquan.content.understanding.service.ContentService;
import com.tzld.piaoquan.content.understanding.service.LoghubService;
import com.tzld.piaoquan.content.understanding.service.ODPSManager;
import com.tzld.piaoquan.content.understanding.service.PipelineService;
import com.tzld.piaoquan.content.understanding.util.DateUtil;
import com.tzld.piaoquan.content.understanding.util.UrlUtil;
import com.xxl.job.core.log.XxlJobLogger;
import lombok.extern.slf4j.Slf4j;
import org.apache.rocketmq.client.apis.message.MessageId;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.time.Duration;
import java.time.LocalDateTime;
import java.util.*;
import java.util.concurrent.TimeUnit;

/**
 * @author supeng
 */
@Slf4j
@Service
public class ContentServiceImpl implements ContentService {

    @Autowired
    private LoghubService loghubService;

    @Autowired
    private PipelineService pipelineService;

    @Value("${video.analyse.pipelineId:1}")
    private Long videoAnalysePipelineId;

    @ApolloJsonValue("${content.understand.pipeline.config:{}}")
    private Map<String, Long> configMap;
    @Autowired
    private CuPipelineMapper cuPipelineMapper;
    @Autowired
    private CuTaskMapper cuTaskMapper;

    private static final Integer MAX_SIZE = 20;

    @Autowired
    private ODPSManager odpsManager;

    @Autowired
    private WxVideoMapper wxVideoMapper;

    @Autowired
    private WxVideoTranscodeMapper wxVideoTranscodeMapper;

    private static final String TRANSCODE_CHANNEL = "WEIZHEN";

    @Value("${video.new.recommend.table:content_ai_tag_recommend}")
    private String newRecommendTable;

    @Value("${video.top.return.table:content_ai_tag_return_top_merge}")
    private String topReturnTable;

    @Value("${video.top.history.table:history_top2000_add_aitags}")
    private String topHistoryTable;

    @Value("${video.analyse:overwrite:0}")
    private String overwriteAnalyse;

    @Value("${old.version.video.understanding.pipelineId:0}")
    private Long oldVersionPipelineId;

    @Autowired
    private AliyunOssManager aliyunOssManager;

    public static final String TRANSCODE_BUCKET = "art-pubbucket";

    private static final Integer EXPIRE_DAYS = 7;

    @Value("${old.version.video.understanding.record.sleep:6}")
    private Long recordSleep;

    @Autowired
    private OldVersionRemainVideoUnderstandingProducer oldVersionRemainVideoUnderstandingProducer;

    @Override
    public void analyse(ContentAnalyseParam param) {
        if (Objects.isNull(param) || Objects.isNull(param.getType())) {
            throw new CommonException(ExceptionEnum.PARAMS_INVALID);
        }
        param.setPipelineId(videoAnalysePipelineId);
        pipelineService.execute(param);
//        try {
//            Long videoId = param.getVideoId();
//            String content = param.getContent();
//            SubmitTasksParam submitTasksParam = new SubmitTasksParam();
//            List<ContentDTO> contentDTOList = new ArrayList<>();
//            ContentDTO contentDTO = new ContentDTO();
//            contentDTO.setVideoId(videoId);
//            contentDTO.setContentType(ContentTypeEnum.VIDEO.getValue());
//            contentDTO.setContent(content);
//            contentDTO.setPipelineId(videoAnalysePipelineId);
//            Map<String, Object> extMap = new HashMap<>();
//            extMap.put("pipelineId", videoAnalysePipelineId);
//            extMap.put("type", "");
//            extMap.put("video_url", content);
//            contentDTO.setExtMap(extMap);
//            contentDTOList.add(contentDTO);
//            submitTasksParam.setContents(contentDTOList);
//            submitTasks(submitTasksParam);
//        } catch (Exception e) {
//            log.error(analyse, e);
//        }
    }

    @Override
    public TaskResultVO submitTasks(SubmitTasksParam param) {
        //参数校验
        if (Objects.isNull(param) || Objects.isNull(param.getContents()) || param.getContents().isEmpty()) {
            throw new CommonException(ExceptionEnum.PARAMS_INVALID);
        }
        List<ContentDTO> contentDTOList = param.getContents();
        //数据校验
        for (ContentDTO contentDTO : contentDTOList) {
            if (Objects.isNull(contentDTO) || Objects.isNull(contentDTO.getPipelineId())
                    || Objects.isNull(contentDTO.getContent()) || Objects.isNull(contentDTO.getVideoId())) {
                throw new CommonException(ExceptionEnum.PARAMS_INVALID, "videoId、pipelineId、content参数不能为空");
            }
            Long pipelineId = contentDTO.getPipelineId();
            CuPipelineExample example = new CuPipelineExample();
            example.createCriteria().andIdEqualTo(pipelineId).andDataStatusEqualTo(DataStatusEnum.VALID.getValue());
            List<CuPipeline> list = cuPipelineMapper.selectByExample(example);
            if (Objects.isNull(list) || list.isEmpty()) {
                throw new CommonException(ExceptionEnum.PARAMS_INVALID, "pipelineId [" + pipelineId + "]无效");
            }
            CuPipeline cuPipeline = list.get(0);
            if (Objects.isNull(cuPipeline) || !ContentTypeEnum.containsValue(cuPipeline.getContentType())) {
                throw new CommonException(ExceptionEnum.PARAMS_INVALID, "pipelineId [" + pipelineId + "]配置异常");
            }
            contentDTO.setContentType(cuPipeline.getContentType());
        }
        //执行任务
        TaskResultVO taskResultVO = new TaskResultVO();
        List<TaskResultDTO> taskResultDTOS = new ArrayList<>();
        for (ContentDTO contentDTO : contentDTOList) {
            TaskResultDTO taskResultDTO = new TaskResultDTO();
            BeanUtils.copyProperties(contentDTO, taskResultDTO);
//            String taskId = pipelineService.executeTask(contentDTO);
            String taskId = pipelineService.submitTask(contentDTO);
            taskResultDTO.setTaskId(taskId);
            taskResultDTOS.add(taskResultDTO);
        }
        taskResultVO.setTaskResults(taskResultDTOS);
        return taskResultVO;
    }

    @Override
    public TaskResultVO queryTasks(QueryTasksParam param) {
        //参数校验
        if (Objects.isNull(param) || Objects.isNull(param.getTaskIds()) || param.getTaskIds().isEmpty()) {
            throw new CommonException(ExceptionEnum.PARAMS_INVALID);
        }
        CuTaskExample example = new CuTaskExample();
        example.createCriteria().andTaskIdIn(param.getTaskIds());
        List<CuTask> list = cuTaskMapper.selectByExample(example);
        TaskResultVO taskResultVO = new TaskResultVO();
        if (Objects.isNull(list) || list.isEmpty()) {
            return taskResultVO;
        }
        List<TaskResultDTO> taskResultDTOS = new ArrayList<>();
        for (CuTask task : list) {
            TaskResultDTO taskResultDTO = new TaskResultDTO();
            BeanUtils.copyProperties(task, taskResultDTO);
            taskResultDTOS.add(taskResultDTO);
        }
        taskResultVO.setTaskResults(taskResultDTOS);
        return taskResultVO;
    }

    @Override
    public void newRecommendVideoUnderstandingHandler() {
        Stopwatch stopwatch = Stopwatch.createStarted();
        String dt = DateUtil.formatLocalDateTime(LocalDateTime.now(), "yyyyMMddHH");
        List<Record> newRecommendRecords = odpsManager.tableTunnelQuery("loghubods", newRecommendTable, "dt=" + dt);
        if (Objects.nonNull(newRecommendRecords) && !newRecommendRecords.isEmpty()) {
            XxlJobLogger.log("table = {} records size = {}", newRecommendTable, newRecommendRecords.size());
            for (Record record : newRecommendRecords) {
                try {
                    Long videoId = record.getBigint("videoid");
                    String type = record.getString("type");
                    String title = record.getString("title");
                    String videoPath = record.getString("video_path");
                    if (Objects.isNull(videoId)) {
                        XxlJobLogger.log("table = {} videoId 为 null", newRecommendTable);
                        continue;
                    }
                    String videoUrl = getVideoUrl(videoId);
                    if (Objects.isNull(videoUrl)) {
                        XxlJobLogger.log("table = {} videoId = {} videoUrl is null", newRecommendTable, videoId);
                        continue;
                    }
                    SubmitTasksParam submitTasksParam = new SubmitTasksParam();
                    List<ContentDTO> contentDTOList = new ArrayList<>();
                    ContentDTO contentDTO = new ContentDTO();
                    contentDTO.setVideoId(videoId);
                    contentDTO.setContentType(ContentTypeEnum.VIDEO.getValue());
                    contentDTO.setContent(videoUrl);
                    contentDTO.setPipelineId(oldVersionPipelineId);
                    //上报日志
//                    1. type 类型
//                    2. video_id 视频ID
//                    3. partition 分区
//                    4. data 数据结果
//                    5. error_msg 错误信息
//                    6. status 状态
//                       success failure
//                    7. pipelineId
//                    8. taskId
//                    9. video_title
//                    10. video_url
                    Map<String, Object> extMap = new HashMap<>();
                    extMap.put("partition", dt);
                    extMap.put("pipelineId", oldVersionPipelineId);
                    extMap.put("type", type);
                    extMap.put("video_title", title);
                    extMap.put("video_url", videoPath);
                    contentDTO.setExtMap(extMap);
                    contentDTOList.add(contentDTO);
                    submitTasksParam.setContents(contentDTOList);
                    submitTasks(submitTasksParam);
                } catch (Exception e) {
                    XxlJobLogger.log("record error {}", e);
                } finally {
                    try {
                        TimeUnit.SECONDS.sleep(recordSleep);
                    } catch (InterruptedException e) {
                        log.error(e.getMessage(), e);
                    }
                }
            }
        } else {
            XxlJobLogger.log("table = {} records is empty", newRecommendTable);
        }
        XxlJobLogger.log("table = {} execute time = {}", newRecommendTable, stopwatch.stop().elapsed(TimeUnit.MILLISECONDS));
    }

    @Override
    public void dayTopVideoUnderstandingHandler() {
        Stopwatch stopwatch = Stopwatch.createStarted();
        String dt = DateUtil.formatLocalDateTime(LocalDateTime.now(), "yyyyMMdd");
        List<Record> topReturnRecords = odpsManager.tableTunnelQuery("loghubods", topReturnTable, "dt=" + dt);
        if (Objects.nonNull(topReturnRecords) && !topReturnRecords.isEmpty()) {
            XxlJobLogger.log("table = {} records size = {}", topReturnTable, topReturnRecords.size());
            for (Record record : topReturnRecords) {
                try {
                    //
                    Long videoId = record.getBigint("videoid");
                    String type = record.getString("type");
                    String title = record.getString("title");
                    String videoPath = record.getString("video_path");
                    if (Objects.isNull(videoId)) {
                        XxlJobLogger.log("table = {} videoId 为 null", topReturnTable);
                        continue;
                    }
                    String videoUrl = getVideoUrl(videoId);
                    if (Objects.isNull(videoUrl)) {
                        XxlJobLogger.log("table = {} videoId = {} videoUrl is null", topReturnTable, videoId);
                        continue;
                    }
                    SubmitTasksParam submitTasksParam = new SubmitTasksParam();
                    List<ContentDTO> contentDTOList = new ArrayList<>();
                    ContentDTO contentDTO = new ContentDTO();
                    contentDTO.setVideoId(videoId);
                    contentDTO.setContentType(ContentTypeEnum.VIDEO.getValue());
                    contentDTO.setContent(videoUrl);
                    contentDTO.setPipelineId(oldVersionPipelineId);
                    //上报日志
//                    1. type 类型
//                    2. video_id 视频ID
//                    3. partition 分区
//                    4. data 数据结果
//                    5. error_msg 错误信息
//                    6. status 状态
//                       success failure
//                    7. pipelineId
//                    8. taskId
//                    9. video_title
//                    10. video_url
                    Map<String, Object> extMap = new HashMap<>();
                    extMap.put("partition", dt);
                    extMap.put("pipelineId", oldVersionPipelineId);
                    extMap.put("type", type);
                    extMap.put("video_title", title);
                    extMap.put("video_url", videoPath);
                    contentDTO.setExtMap(extMap);
                    contentDTOList.add(contentDTO);
                    submitTasksParam.setContents(contentDTOList);
                    TaskResultVO taskResultVO = submitTasks(submitTasksParam);
                } catch (Exception e) {
                    XxlJobLogger.log("record = {} error", record);
                } finally {
                    try {
                        TimeUnit.SECONDS.sleep(recordSleep);
                    } catch (InterruptedException e) {
                        log.error(e.getMessage(), e);
                    }
                }
            }
        } else {
            XxlJobLogger.log("table = {} records is empty", topReturnTable);
        }
        XxlJobLogger.log("table = {} execute time = {}", topReturnTable, stopwatch.stop().elapsed(TimeUnit.MILLISECONDS));
    }

    @Override
    public void historyVideoUnderstandingHandler() {
        Stopwatch stopwatch = Stopwatch.createStarted();
        String dt = DateUtil.formatLocalDateTime(LocalDateTime.now(), "yyyyMMdd");
        List<Record> topHistoryRecords = odpsManager.tableTunnelQuery("loghubods", topHistoryTable, "dt=" + dt);
        if (Objects.nonNull(topHistoryRecords) && !topHistoryRecords.isEmpty()) {
            XxlJobLogger.log("table = {} records size = {}", topHistoryTable, topHistoryRecords.size());
            for (Record record : topHistoryRecords) {
                try {
                    Long videoId = record.getBigint("videoid");
                    String type = record.getString("type");
                    String title = record.getString("title");
                    String videoPath = record.getString("video_path");
                    if (Objects.isNull(videoId)) {
                        XxlJobLogger.log("table = {} videoId 为 null", topHistoryTable);
                        continue;
                    }
                    String videoUrl = getVideoUrl(videoId);
                    if (Objects.isNull(videoUrl)) {
                        XxlJobLogger.log("table = {} videoId = {} videoUrl is null", topHistoryTable, videoId);
                        continue;
                    }
                    SubmitTasksParam submitTasksParam = new SubmitTasksParam();
                    List<ContentDTO> contentDTOList = new ArrayList<>();
                    ContentDTO contentDTO = new ContentDTO();
                    contentDTO.setVideoId(videoId);
                    contentDTO.setContentType(ContentTypeEnum.VIDEO.getValue());
                    contentDTO.setContent(videoUrl);
                    contentDTO.setPipelineId(oldVersionPipelineId);
                    //上报日志
//                    1. type 类型
//                    2. video_id 视频ID
//                    3. partition 分区
//                    4. data 数据结果
//                    5. error_msg 错误信息
//                    6. status 状态
//                       success failure
//                    7. pipelineId
//                    8. taskId
//                    9. video_title
//                    10. video_url
                    Map<String, Object> extMap = new HashMap<>();
                    extMap.put("partition", dt);
                    extMap.put("pipelineId", oldVersionPipelineId);
                    extMap.put("type", type);
                    extMap.put("video_title", title);
                    extMap.put("video_url", videoPath);
                    contentDTO.setExtMap(extMap);
                    contentDTOList.add(contentDTO);
                    submitTasksParam.setContents(contentDTOList);
                    TaskResultVO taskResultVO = submitTasks(submitTasksParam);
                } catch (Exception e) {
                    XxlJobLogger.log("record = {} error", record);
                } finally {
                    try {
                        TimeUnit.SECONDS.sleep(recordSleep);
                    } catch (InterruptedException e) {
                        log.error(e.getMessage(), e);
                    }
                }
            }
        } else {
            XxlJobLogger.log("table = {} records is empty", topHistoryTable);
        }
        XxlJobLogger.log("table = {} execute time = {}", topHistoryTable, stopwatch.stop().elapsed(TimeUnit.MILLISECONDS));
    }

    /**
     * 获取视频URL
     * <p>
     * 有多码率文件使用多码率文件（多码率文件目前不会被冻结/冷归档/深度冷归档和删除）
     * 没有多码率文件使用转码文件，转码文件被冻结/冷归档/深度冷归档，则需要解冻
     * 没有转码文件使用原文件，原文件被冻结/冷归档/深度冷归档，则需要解冻
     * 没有原文件则失败
     *
     * @param videoId
     * @return
     */
    private String getVideoUrl(Long videoId) {
        //获取多码率
        String videoUrl = null;
        boolean isM3u8 = false;
        WxVideoTranscodeExample transcodeExample = new WxVideoTranscodeExample();
        transcodeExample.createCriteria().andVideoIdEqualTo(videoId).andTransChannelEqualTo(TRANSCODE_CHANNEL)
                .andTranscodeStatusEqualTo(3);
        List<WxVideoTranscode> transcodeList = wxVideoTranscodeMapper.selectByExample(transcodeExample);
        if (Objects.nonNull(transcodeList) && !transcodeList.isEmpty()) {
            WxVideoTranscode wxVideoTranscode = transcodeList.get(0);
            String transcodeVideoPath = wxVideoTranscode.getTranscodeVideoPath();
            if (Objects.nonNull(transcodeVideoPath)) {
                isM3u8 = UrlUtil.isM3u8ByExtension(transcodeVideoPath);
                if (!isM3u8) {
                    videoUrl = UrlUtil.processUrl(Constant.MULTI_CDN_HTTP_DOMAIN, transcodeVideoPath);
                }
            }
        }
        //没有多码率文件
        if (Objects.isNull(videoUrl) && !isM3u8) {
            WxVideo wxVideo = wxVideoMapper.selectByPrimaryKey(videoId);
            if (Objects.isNull(wxVideo)) {
                XxlJobLogger.log("table = {} videoId = {} wxVideo is null", topReturnTable, videoId);
                return null;
            }
            String transedVideoPath = wxVideo.getTransedVideoPath();
            if (Objects.nonNull(transedVideoPath) && !transedVideoPath.isEmpty()) {
                ObjectMetadata objectMetadata = aliyunOssManager.getObjectMetadata(TRANSCODE_BUCKET, transedVideoPath);
                if (Objects.nonNull(objectMetadata)) {
                    isM3u8 = UrlUtil.isM3u8ByExtension(transedVideoPath);
                    if (!isM3u8) {
                        videoUrl = UrlUtil.processUrl(Constant.VIDEO_CDN_HTTP_DOMAIN, transedVideoPath);
                        boolean isCodeArchive = Objects.equals(objectMetadata.getObjectStorageClass(), StorageClass.ColdArchive);
                        boolean isDeepColdArchive = Objects.equals(objectMetadata.getObjectStorageClass(), StorageClass.DeepColdArchive);
                        if (isCodeArchive || isDeepColdArchive) {
                            try {
                                //throw NPE 未解冻 true 解冻完成 false 解冻中
                                if (objectMetadata.isRestoreCompleted()) {
                                    //解冻完成
                                    return videoUrl;
                                } else {
                                    //解冻中
                                    return null;
                                }
                            } catch (Exception e) {
                                //未开始解冻
                                log.error("{} {} not restoreObject", videoId, videoUrl);
                            }
                            //解冻
                            aliyunOssManager.restoreObject(TRANSCODE_BUCKET, transedVideoPath, EXPIRE_DAYS);
//                            Map<String, Object> messageMap = new HashMap<>();
//                            messageMap.put("videoId", videoId);
                            VideoUnderstandingMessageDTO messageDTO = new VideoUnderstandingMessageDTO();
                            messageDTO.setVideoId(videoId);
                            Duration duration;
                            if (isCodeArchive) {
                                duration = Duration.ofHours(1);
                            } else {
                                duration = Duration.ofHours(12);
                            }

                            try {
                                MessageId messageId = oldVersionRemainVideoUnderstandingProducer.syncSendDelayMessage(messageDTO, duration);
                                log.info("syncSendDelayMessage messageId = {}", messageId);
                            } catch (Exception e) {
                                log.error("syncSendDelayMessage error messageDTO = {}", e, messageDTO);
                            }
                            return null;
                        } else {
                            return videoUrl;
                        }
                    }
                }
            }
        }
        //没有转码文件
        if (Objects.isNull(videoUrl)) {
            WxVideo wxVideo = wxVideoMapper.selectByPrimaryKey(videoId);
            if (Objects.isNull(wxVideo)) {
                XxlJobLogger.log("table = {} videoId = {} wxVideo is null", topReturnTable, videoId);
                return null;
            }
            String videoPath = wxVideo.getVideoPath();
            if (Objects.nonNull(videoPath) && !videoPath.isEmpty()) {
                ObjectMetadata objectMetadata = aliyunOssManager.getObjectMetadata(TRANSCODE_BUCKET, videoPath);
                if (Objects.nonNull(objectMetadata)) {
                    videoUrl = UrlUtil.processUrl(Constant.VIDEO_CDN_HTTP_DOMAIN, videoPath);
                    boolean isCodeArchive = Objects.equals(objectMetadata.getObjectStorageClass(), StorageClass.ColdArchive);
                    boolean isDeepColdArchive = Objects.equals(objectMetadata.getObjectStorageClass(), StorageClass.ColdArchive);
                    if (isCodeArchive || isDeepColdArchive) {
                        try {
                            //throw NPE 未解冻 true 解冻完成 false 解冻中
                            if (objectMetadata.isRestoreCompleted()) {
                                //解冻完成
                                return videoUrl;
                            } else {
                                //解冻中
                                return null;
                            }
                        } catch (Exception e) {
                            log.error("{} {} not restoreObject", videoId, videoUrl);
                        }
                        //解冻
                        aliyunOssManager.restoreObject(TRANSCODE_BUCKET, videoPath, EXPIRE_DAYS);
//                        Map<String, Object> messageMap = new HashMap<>();
//                        messageMap.put("videoId", videoId);
                        VideoUnderstandingMessageDTO messageDTO = new VideoUnderstandingMessageDTO();
                        messageDTO.setVideoId(videoId);
                        Duration duration;
                        if (isCodeArchive) {
                            duration = Duration.ofHours(1);
                        } else {
                            duration = Duration.ofHours(12);
                        }
                        try {
                            oldVersionRemainVideoUnderstandingProducer.syncSendDelayMessage(messageDTO, duration);
                        } catch (Exception e) {
                            log.error("syncSendDelayMessage error messageDTO = {}", e, messageDTO);
                        }
                        return null;
                    } else {
                        return videoUrl;
                    }
                }
            }
        }
        return videoUrl;
    }

    @Override
    public void remainVideoUnderstandingHandler() {

    }

    @Override
    public boolean remainVideoUnderstandingHandler(Long videoId) {
        try {
            String videoUrl = getVideoUrl(videoId);
            if (Objects.isNull(videoUrl)) {
                XxlJobLogger.log("table = {} videoId = {} videoUrl is null", newRecommendTable, videoId);
                return false;
            }
            SubmitTasksParam submitTasksParam = new SubmitTasksParam();
            List<ContentDTO> contentDTOList = new ArrayList<>();
            ContentDTO contentDTO = new ContentDTO();
            contentDTO.setVideoId(videoId);
            contentDTO.setContentType(ContentTypeEnum.VIDEO.getValue());
            contentDTO.setContent(videoUrl);
            contentDTO.setPipelineId(oldVersionPipelineId);
            contentDTOList.add(contentDTO);
            submitTasksParam.setContents(contentDTOList);
            TaskResultVO taskResultVO = submitTasks(submitTasksParam);
            if (Objects.nonNull(taskResultVO) && Objects.nonNull(taskResultVO.getTaskResults())
                    && taskResultVO.getTaskResults().size() > 0 && Objects.nonNull(taskResultVO.getTaskResults().get(0))
                    && Objects.nonNull(taskResultVO.getTaskResults().get(0).getTaskId())) {
                return true;
            }
        } catch (Exception e) {
            log.error("remainVideoUnderstandingHandler error", e);
        }
        return false;
    }
}
