package com.tzld.piaoquan.content.understanding.service.impl;

import com.alibaba.fastjson.JSON;
import com.tzld.piaoquan.content.understanding.common.base.CommonResponse;
import com.tzld.piaoquan.content.understanding.common.enums.ContentTypeEnum;
import com.tzld.piaoquan.content.understanding.common.enums.ResponseFormatTypeEnum;
import com.tzld.piaoquan.content.understanding.model.param.ActionParam;
import com.tzld.piaoquan.content.understanding.model.param.DeepSeekParam;
import com.tzld.piaoquan.content.understanding.model.param.GeminiParam;
import com.tzld.piaoquan.content.understanding.service.Action;
import com.tzld.piaoquan.content.understanding.util.HttpClientUtil;
import com.tzld.piaoquan.content.understanding.util.HttpPoolClient;
import com.tzld.piaoquan.content.understanding.util.RetryUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.concurrent.ThreadLocalRandom;

/**
 * @author supeng
 */
@Slf4j
@Service(value = "deepSeekGenerateContentAction")
public class DeepSeekGenerateContentAction implements Action {

    @Value("${ai.deepSeekApi.url:http://ai-api-internal.piaoquantv.com/aigc-server/deepseek/generateContent}")
    private String deepSeekApiUrl;

    @Value("#{'${deepseek.apikey.list:[]}'.split(',')}")
    private List<String> apiKeyList;

    private static final HttpPoolClient httpPoolClient = HttpClientUtil.create(3000, 900000, 50, 200, 0, 3000);

    public static final String MODEL_CHAT = "deepseek-chat";
    public static final String MODEL_REASONER = "deepseek-reasoner";
    public static final Double TEMPERATURE = 1.0;

    private static final Integer RETRY_TIMES = 3;

    @Override
    public String execute(ActionParam param) {
        Integer type = param.getType();
        //类型校验
        if (!Objects.equals(ContentTypeEnum.TITLE.getValue(), type) && !Objects.equals(ContentTypeEnum.INTRODUCTION.getValue(), type)) {
            log.error("deepseek 不支持非文本类型 param = {}", param);
            return null;
        }
        ThreadLocalRandom random = ThreadLocalRandom.current();
        int index = random.nextInt(apiKeyList.size());
        String apiKey = apiKeyList.get(index);
        DeepSeekParam deepSeekParam = new DeepSeekParam();
        deepSeekParam.setApiKey(apiKey);
        deepSeekParam.setPrompt(param.getPrompt());
        deepSeekParam.setModel(MODEL_REASONER);
        deepSeekParam.setTemperature(TEMPERATURE);
        //reasonser模型不支持 json format
//        deepSeekParam.setResponseFormat(ResponseFormatTypeEnum.JSON.getValue());
        return RetryUtil.executeWithRetry(() -> {
            try {
                log.info("deepSeekGenerateContentAction deepSeekParam = {}", JSON.toJSONString(deepSeekParam));
                Optional<String> optionalS = httpPoolClient.postJson(deepSeekApiUrl, JSON.toJSONString(deepSeekParam));
                log.info("deepSeekGenerateContentAction optionalS = {}", optionalS);
                if (optionalS.isPresent()) {
                    CommonResponse<Map<String, Object>> commonResponse = JSON.parseObject(optionalS.get(), CommonResponse.class);
                    if (commonResponse.isSuccess() && Objects.nonNull(commonResponse.getData()) && Objects.nonNull(commonResponse.getData().get("result"))) {
                        String content = commonResponse.getData().get("result").toString();
                        //过滤一些特殊格式
                        content = content.replace("```json", "").replace("```", "");
                        return content;
                    } else {
                        throw new RuntimeException("deepSeekGenerateContentAction request failure geminiParam = " + deepSeekParam);
                    }
                } else {
                    throw new RuntimeException("deepSeekGenerateContentAction optionalS is null geminiParam = " + deepSeekParam);
                }
            } catch (Exception e) {
                log.error("deepSeekGenerateContentAction error deepSeekParam = {}", deepSeekParam);
                throw new RuntimeException("deepSeekGenerateContentAction request error", e);
            }
        }, RETRY_TIMES, "deepSeekGenerateContentAction", i -> (long) (Math.pow(5, i) * 1000));
    }
}
