package com.tzld.piaoquan.content.understanding.service.impl;

import com.google.common.util.concurrent.ThreadFactoryBuilder;
import com.tzld.commons.aliyun.log.AliyunLogManager;
import com.tzld.piaoquan.content.understanding.service.LoghubService;
import lombok.extern.slf4j.Slf4j;
import org.checkerframework.checker.units.qual.A;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.annotation.PostConstruct;
import javax.annotation.PreDestroy;
import java.util.Map;
import java.util.concurrent.*;

/**
 * @author supeng
 */
@Slf4j
@Service
public class LoghubServiceImpl implements LoghubService {

    @Autowired
    private AliyunLogManager aliyunLogManager;

    /**
     * 线程池队列大小
     */
    private static final int QUEUE_MAX_SIZE = 100000;
    /**
     * 线程命名
     */
    private static final ThreadFactory NAMED_THREAD_FACTORY = new ThreadFactoryBuilder().setNameFormat("loghub-service-pool-%d").build();
    /**
     * 线程池
     */
    private static ExecutorService pool;

    @PostConstruct
    public void init() {
        //init thread pool
        pool = new ThreadPoolExecutor(32, 32,
                0L, TimeUnit.MILLISECONDS, new LinkedBlockingQueue<>(QUEUE_MAX_SIZE), NAMED_THREAD_FACTORY, new ThreadPoolExecutor.AbortPolicy());
    }

    @PreDestroy
    public void destroy() {
        //gracefully shutdown
        pool.shutdown();
    }

    @Override
    public void asyncSubmitLog(String project, String logStore, String topic, Map<String, Object> data) {
        pool.execute(() -> aliyunLogManager.sendLog(project, logStore, topic, data));
    }
}
